@@ -9,13 +9,9 @@
 #include "deferdlg.h"
 
 #include "alarmtimewidget.h"
-#include "functions.h"
-#include "kalarmapp.h"
-#include "resourcescalendar.h"
 #include "resources/resources.h"
 #include "lib/messagebox.h"
 #include "kalarmcalendar/kaevent.h"
-#include "kalarm_debug.h"
 
 #include <KLocalizedString>
 
@@ -114,7 +110,7 @@ void DeferAlarmDlg::slotOk()
             case KAEvent::LIMIT_NONE:
                 break;   // can't happen with a valid endTime
         }
-        KAMessageBox::sorry(this, text);
+        KAMessageBox::error(this, text);
     }
     else
         accept();
