@@ -1,25 +1,23 @@
 /*
  *  fontcolour.cpp  -  font and colour chooser widget
  *  Program:  kalarm
- *  SPDX-FileCopyrightText: 2001-2020 David Jarvie <djarvie@kde.org>
+ *  SPDX-FileCopyrightText: 2001-2022 David Jarvie <djarvie@kde.org>
  *
  *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "fontcolour.h"
 
-#include "kalarmapp.h"
-#include "preferences.h"
 #include "lib/colourbutton.h"
 #include "lib/checkbox.h"
 
 #include <KFontChooser>
-#include <kwidgetsaddons_version.h>
 
+#include <QEvent>
 #include <QGroupBox>
 #include <QLabel>
-#include <QVBoxLayout>
 #include <QHBoxLayout>
+#include <QVBoxLayout>
 
 
 FontColourChooser::FontColourChooser(QWidget* parent, const QStringList& fontList,
@@ -100,16 +98,10 @@ FontColourChooser::FontColourChooser(QWidget* parent, const QStringList& fontLis
     slotDefaultFontToggled(false);
 }
 
-void FontColourChooser::setDefaultFont()
+void FontColourChooser::setFont(const QFont& font, bool defaultFont, bool onlyFixed)
 {
     if (mDefaultFont)
-        mDefaultFont->setChecked(true);
-}
-
-void FontColourChooser::setFont(const QFont& font, bool onlyFixed)
-{
-    if (mDefaultFont)
-        mDefaultFont->setChecked(false);
+        mDefaultFont->setChecked(defaultFont);
     mFontChooser->setFont(font, onlyFixed);
 }
 
