@@ -54,7 +54,9 @@ using namespace KCalendarCore;
 #include <KIO/StoredTransferJob>
 #include <KFileCustomDialog>
 #include <KWindowInfo>
-#include <KWindowSystem>
+#if ENABLE_X11
+#include <KX11Extras>
+#endif
 
 #include <QAction>
 #include <QDBusConnectionInterface>
@@ -65,7 +67,6 @@ using namespace KCalendarCore;
 #include <QStandardPaths>
 #include <QPushButton>
 #include <QTemporaryFile>
-#include <kwidgetsaddons_version.h>
 
 //clazy:excludeall=non-pod-global-static
 
@@ -162,15 +163,17 @@ MainWindow* displayMainWindowSelected(const QString& eventId)
     else
     {
         // There is already a main window, so make it the active window
+#if ENABLE_X11
         KWindowInfo wi(win->winId(), NET::WMDesktop);
-        if (!wi.valid()  ||  !wi.isOnDesktop(KWindowSystem::currentDesktop()))
+        if (!wi.valid()  ||  !wi.isOnDesktop(KX11Extras::currentDesktop()))
         {
-            // The main window isn't on the current desktop. Hide it first so
-            // that it will be shown on the current desktop when it is shown
-            // again. Note that this shifts the window's position, so don't
-            // hide it if it's already on the current desktop.
+            // The main window isn't on the current virtual desktop. Hide it
+            // first so that it will be shown on the current desktop when it is
+            // shown again. Note that this shifts the window's position, so
+            // don't hide it if it's already on the current desktop.
             win->hide();
         }
+#endif
         win->setWindowState(win->windowState() & ~Qt::WindowMinimized);
         win->show();
         win->raise();
@@ -1611,11 +1614,7 @@ void outputAlarmWarnings(QWidget* parent, const KAEvent* event)
         if (KAMessageBox::warningYesNo(parent, xi18nc("@info", "<para>Alarms are currently disabled.</para><para>Do you want to enable alarms now?</para>"),
                                        QString(), KGuiItem(i18nc("@action:button", "Enable")), KGuiItem(i18nc("@action:button", "Keep Disabled")),
                                        QStringLiteral("EditEnableAlarms"))
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
                         == KMessageBox::ButtonCode::PrimaryAction)
-#else
-                        == KMessageBox::Yes)
-#endif
             theApp()->setAlarmsEnabled(true);
     }
 }
