@@ -30,10 +30,10 @@
 #include <KLocalizedString>
 #include <KConfigGroup>
 #include <KTextEdit>
-#include <KWindowSystem>
 #include <KSqueezedTextLabel>
-#include <KIdleTime>
+#include <KWindowSystem>
 #if ENABLE_X11
+#include <KX11Extras>
 #include <KWindowInfo>
 #include <netwm.h>
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
@@ -518,7 +518,9 @@ void MessageWindow::setUpDisplay()
         wstate |= NET::Modal;
     WId winid = winId();
     KWindowSystem::setState(winid, wstate);
-    KWindowSystem::setOnAllDesktops(winid, true);
+#if ENABLE_X11
+    KX11Extras::setOnAllDesktops(winid, true);   // show on all virtual desktops
+#endif
 
     mInitialised = true;   // the window's widgets have been created
 }
@@ -819,7 +821,7 @@ void MessageWindow::display()
         MainWindowBase::show();
         // Ensure that the screen wakes from sleep, in case the window manager
         // doesn't do this when the window is displayed.
-        KIdleTime::instance()->simulateUserActivity();
+        mHelper->wakeScreen();
     }
 }
 
@@ -1069,11 +1071,7 @@ bool MessageWindow::confirmAcknowledgement()
         // Ask for confirmation of acknowledgement. Use warningYesNo() because its default is No.
         if (KAMessageBox::warningYesNo(this, i18nc("@info", "Do you really want to acknowledge this alarm?"),
                                              i18nc("@action:button", "Acknowledge Alarm"), KGuiItem(i18nc("@action:button", "Acknowledge")), KStandardGuiItem::cancel())
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
             != KMessageBox::ButtonCode::PrimaryAction)
-#else
-            != KMessageBox::Yes)
-#endif
         {
             return false;
         }
@@ -1147,9 +1145,13 @@ void MessageWindow::slotEdit()
     if (!dlg)
         return;
     KWindowSystem::setMainWindow(dlg->windowHandle(), winId());
-    KWindowSystem::setOnAllDesktops(dlg->winId(), false);
+#if ENABLE_X11
+    KX11Extras::setOnAllDesktops(dlg->winId(), false);   // don't show on all virtual desktops
+#endif
     setButtonsReadOnly(true);
-    connect(KWindowSystem::self(), &KWindowSystem::activeWindowChanged, this, &MessageWindow::activeWindowChanged);
+#if ENABLE_X11
+    connect(KX11Extras::self(), &KX11Extras::activeWindowChanged, this, &MessageWindow::activeWindowChanged);
+#endif
     mHelper->executeEdit();
 }
 
@@ -1169,8 +1171,12 @@ void MessageWindow::editDlgCancelled()
 */
 void MessageWindow::activeWindowChanged(WId win)
 {
+#if ENABLE_X11
+    // Note that the alarm edit dialog is not a QWindow, so we can't call
+    // QWindow::requestActivate().
     if (mEditDlg()  &&  win == winId())
-        KWindowSystem::activateWindow(mEditDlg()->winId());
+        KX11Extras::activateWindow(mEditDlg()->winId());
+#endif
 }
 
 /******************************************************************************
@@ -1320,10 +1326,12 @@ bool MessageWindow::getWorkAreaAndModal()
     }
     if (modal)
     {
-        const WId activeId = KWindowSystem::activeWindow();
+#if ENABLE_X11
+        const WId activeId = KX11Extras::activeWindow();
         const KWindowInfo wi = KWindowInfo(activeId, NET::WMState);
         if (wi.valid()  &&  wi.hasState(NET::FullScreen))
             return false;    // the active window is full screen.
+#endif
     }
     return modal;
 }
@@ -1388,7 +1396,7 @@ FullScreenType findFullScreenWindows(const QVector<QRect>& screenRects, QVector<
         using namespace QNativeInterface;
         auto* x11App = qGuiApp->nativeInterface<QX11Application>();
         if (!x11App)
-            return type;
+            return result;
         xcb_connection_t* connection = x11App->connection();
 #else
         xcb_connection_t* connection = QX11Info::connection();
