@@ -13,7 +13,6 @@
 #include "editdlg.h"
 #include "editdlgtypes.h"
 #include "fontcolour.h"
-#include "functions.h"
 #include "kalarmapp.h"
 #include "kamail.h"
 #include "latecancel.h"
@@ -25,7 +24,6 @@
 #include "sounddlg.h"
 #include "soundpicker.h"
 #include "specialactions.h"
-#include "traywindow.h"
 #include "resources/resources.h"
 #include "lib/buttongroup.h"
 #include "lib/checkbox.h"
@@ -48,9 +46,8 @@
 #include <KHolidays/HolidayRegion>
 using namespace KHolidays;
 
-#include <kpimtextedit/kpimtextedit-texttospeech.h>
-#if KPIMTEXTEDIT_TEXT_TO_SPEECH
-#include <KPIMTextEdit/TextToSpeech>
+#ifdef HAVE_TEXT_TO_SPEECH_SUPPORT
+#include <TextEditTextToSpeech/TextToSpeech>
 #endif
 
 #include <KLocalizedString>
@@ -59,11 +56,10 @@ using namespace KHolidays;
 #include <KStandardGuiItem>
 #include <QIcon>
 #if ENABLE_X11
-#include <KWindowInfo>
+#include <KX11Extras>
 #endif
 #include <KWindowSystem>
 #include <KHelpClient>
-#include <kwidgetsaddons_version.h>
 
 #include <QLabel>
 #include <QCheckBox>
@@ -87,23 +83,6 @@ using namespace KAlarmCal;
 
 static const char PREF_DIALOG_NAME[] = "PrefDialog";
 
-// Command strings for executing commands in different types of terminal windows.
-// %t = window title parameter
-// %c = command to execute in terminal
-// %w = command to execute in terminal, with 'sleep 86400' appended
-// %C = temporary command file to execute in terminal
-// %W = temporary command file to execute in terminal, with 'sleep 86400' appended
-static QString xtermCommands[] = {
-    QStringLiteral("xterm -sb -hold -title %t -e %c"),
-    QStringLiteral("konsole --noclose -p tabtitle=%t -e ${SHELL:-sh} -c %c"),
-    QStringLiteral("gnome-terminal -t %t -e %W"),
-    QStringLiteral("eterm --pause -T %t -e %C"),    // some systems use eterm...
-    QStringLiteral("Eterm --pause -T %t -e %C"),    // while some use Eterm
-    QStringLiteral("rxvt -title %t -e ${SHELL:-sh} -c %w"),
-    QStringLiteral("xfce4-terminal -T %t -H -e %c"),
-    QString()       // end of list indicator - don't change!
-};
-
 
 /*=============================================================================
 = Class KAlarmPrefDlg
@@ -112,6 +91,9 @@ static QString xtermCommands[] = {
 KAlarmPrefDlg*         KAlarmPrefDlg::mInstance = nullptr;
 KAlarmPrefDlg::TabType KAlarmPrefDlg::mLastTab  = AnyTab;
 
+/******************************************************************************
+* Display the Preferences dialog as a non-modal window.
+*/
 void KAlarmPrefDlg::display()
 {
     if (!mInstance)
@@ -126,9 +108,10 @@ void KAlarmPrefDlg::display()
     else
     {
         mInstance->restoreTab();
+        // Switch to the virtual desktop which the dialog is in
 #if ENABLE_X11
         KWindowInfo info = KWindowInfo(mInstance->winId(), NET::WMGeometry | NET::WMDesktop);
-        KWindowSystem::setCurrentDesktop(info.desktop());
+        KX11Extras::setCurrentDesktop(info.desktop());
 #endif
         mInstance->setWindowState(mInstance->windowState() & ~Qt::WindowMinimized); // un-minimize it if necessary
         mInstance->raise();
@@ -185,12 +168,14 @@ KAlarmPrefDlg::KAlarmPrefDlg()
     mViewPageItem->setHeader(i18nc("@title", "View Settings"));
     mViewPageItem->setIcon(QIcon::fromTheme(QStringLiteral("preferences-desktop-theme")));
     addPage(mViewPageItem);
+
     connect(button(QDialogButtonBox::Ok), &QAbstractButton::clicked, this, &KAlarmPrefDlg::slotOk);
     connect(button(QDialogButtonBox::Cancel), &QAbstractButton::clicked, this, &KAlarmPrefDlg::slotCancel);
     connect(button(QDialogButtonBox::Apply), &QAbstractButton::clicked, this, &KAlarmPrefDlg::slotApply);
     connect(button(QDialogButtonBox::RestoreDefaults), &QAbstractButton::clicked, this, &KAlarmPrefDlg::slotDefault);
     connect(button(QDialogButtonBox::Help), &QAbstractButton::clicked, this, &KAlarmPrefDlg::slotHelp);
     connect(this, &KPageDialog::currentPageChanged, this, &KAlarmPrefDlg::slotTabChanged);
+
     restore(false);
     adjustSize();
 }
@@ -209,15 +194,18 @@ void KAlarmPrefDlg::slotHelp()
 void KAlarmPrefDlg::slotApply()
 {
     qCDebug(KALARM_LOG) << "KAlarmPrefDlg::slotApply";
+    mValid = true;
+    apply();
+}
+
+// Apply the preferences that are currently selected
+void KAlarmPrefDlg::apply()
+{
     QString errmsg = mEmailPage->validate();
     if (!errmsg.isEmpty())
     {
         setCurrentPage(mEmailPageItem);
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
         if (KAMessageBox::warningYesNo(this, errmsg) != KMessageBox::ButtonCode::PrimaryAction)
-#else
-        if (KAMessageBox::warningYesNo(this, errmsg) != KMessageBox::Yes)
-#endif
         {
             mValid = false;
             return;
@@ -231,13 +219,12 @@ void KAlarmPrefDlg::slotApply()
         mValid = false;
         return;
     }
-    mValid = true;
-    mEmailPage->apply(false);
-    mViewPage->apply(false);
-    mEditPage->apply(false);
-    mStorePage->apply(false);
-    mTimePage->apply(false);
-    mMiscPage->apply(false);
+    mValid = mEmailPage->apply(false) && mValid;
+    mValid = mViewPage->apply(false)  && mValid;
+    mValid = mEditPage->apply(false)  && mValid;
+    mValid = mStorePage->apply(false) && mValid;
+    mValid = mTimePage->apply(false)  && mValid;
+    mValid = mMiscPage->apply(false)  && mValid;
     Preferences::self()->save();
 }
 
@@ -246,11 +233,17 @@ void KAlarmPrefDlg::slotOk()
 {
     qCDebug(KALARM_LOG) << "KAlarmPrefDlg::slotOk";
     mValid = true;
-    slotApply();
+    apply();
     if (mValid)
         QDialog::accept();
 }
 
+void KAlarmPrefDlg::accept()
+{
+    // accept() is called automatically by KPageDialog when OK button is clicked.
+    // Don't accept and close the dialog until slotOk() has finished processing.
+}
+
 // Discard the current preferences and close the dialog
 void KAlarmPrefDlg::slotCancel()
 {
@@ -268,18 +261,10 @@ void KAlarmPrefDlg::slotDefault()
                          KGuiItem(i18nc("@action:button Reset ALL tabs", "&All")),
                          KGuiItem(i18nc("@action:button Reset the CURRENT tab", "C&urrent"))))
     {
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
         case KMessageBox::ButtonCode::PrimaryAction:
-#else
-        case KMessageBox::Yes:
-#endif
             restore(true);   // restore all tabs
             break;
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
         case KMessageBox::ButtonCode::SecondaryAction:
-#else
-        case KMessageBox::No:
-#endif
             Preferences::self()->useDefaults(true);
             static_cast<PrefsTabBase*>(currentPage()->widget())->restore(true, false);
             Preferences::self()->useDefaults(false);
@@ -404,10 +389,11 @@ PrefsTabBase::PrefsTabBase(StackedScrollGroup* scrollGroup)
     }
 }
 
-void PrefsTabBase::apply(bool syncToDisc)
+bool PrefsTabBase::apply(bool syncToDisc)
 {
     if (syncToDisc)
         Preferences::self()->save();
+    return true;
 }
 
 void PrefsTabBase::addAlignedLabel(QLabel* label)
@@ -508,18 +494,14 @@ MiscPrefTab::MiscPrefTab(StackedScrollGroup* scrollGroup)
 
     mXtermType = new ButtonGroup(group);
     int index = 0;
-    mXtermFirst = -1;
-    for (mXtermCount = 0;  !xtermCommands[mXtermCount].isNull();  ++mXtermCount)
+    const auto xtermCommands = Preferences::cmdXTermStandardCommands();
+    for (auto it = xtermCommands.constBegin();  it != xtermCommands.constEnd();  ++it, ++index)
     {
-        QString cmd = xtermCommands[mXtermCount];
+        QString cmd = it.value();
         const QStringList args = KShell::splitArgs(cmd);
-        if (args.isEmpty()  ||  QStandardPaths::findExecutable(args[0]).isEmpty())
-            continue;
         auto radio = new QRadioButton(args[0], group);
         radio->setMinimumSize(radio->sizeHint());
-        mXtermType->addButton(radio, mXtermCount);
-        if (mXtermFirst < 0)
-            mXtermFirst = mXtermCount;   // note the id of the first button
+        mXtermType->addButton(radio, it.key());
         cmd.replace(QStringLiteral("%t"), KAboutData::applicationData().displayName());
         cmd.replace(QStringLiteral("%c"), QStringLiteral("<command>"));
         cmd.replace(QStringLiteral("%w"), QStringLiteral("<command; sleep>"));
@@ -528,7 +510,6 @@ MiscPrefTab::MiscPrefTab(StackedScrollGroup* scrollGroup)
         radio->setWhatsThis(
                 xi18nc("@info:whatsthis", "Check to execute command alarms in a terminal window by <icode>%1</icode>", cmd));
         grid->addWidget(radio, (row = index/3), index % 3, Qt::AlignLeft);
-        ++index;
     }
 
     // QHBox used here doesn't allow the QLineEdit to expand!?
@@ -537,9 +518,8 @@ MiscPrefTab::MiscPrefTab(StackedScrollGroup* scrollGroup)
     QRadioButton* radio = new QRadioButton(i18nc("@option:radio Other terminal window command", "Other:"), group);
     hlayout->addWidget(radio);
     connect(radio, &QAbstractButton::toggled, this, &MiscPrefTab::slotOtherTerminalToggled);
-    mXtermType->addButton(radio, mXtermCount);
-    if (mXtermFirst < 0)
-        mXtermFirst = mXtermCount;   // note the id of the first button
+    mXtermType->addButton(radio, 0);
+
     mXtermCommand = new QLineEdit(group);
     mXtermCommand->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
     hlayout->addWidget(mXtermCommand);
@@ -576,32 +556,54 @@ void MiscPrefTab::restore(bool defaults, bool)
     mUseAlarmNames->setChecked(Preferences::useAlarmName());
     mConfirmAlarmDeletion->setChecked(Preferences::confirmAlarmDeletion());
     mDefaultDeferTime->setValue(Preferences::defaultDeferTime());
-    QString xtermCmd = Preferences::cmdXTermCommand();
-    int id = mXtermFirst;
-    if (!xtermCmd.isEmpty())
+    const auto xtermCmd = Preferences::cmdXTermCommandIndex();
+    int id = xtermCmd.first;
+    if (id > 0  &&  !mXtermType->find(id))
     {
-        for ( ;  id < mXtermCount;  ++id)
-        {
-            if (mXtermType->find(id)  &&  xtermCmd == xtermCommands[id])
-                break;
-        }
+        // The command is a standard command, but there is no button for it
+        // (because the executable doesn't exist on this system). So set it
+        // into the 'Other' option.
+        id = 0;
     }
     mXtermType->setButton(id);
-    mXtermCommand->setEnabled(id == mXtermCount);
-    mXtermCommand->setText(id == mXtermCount ? xtermCmd : QString());
+    mXtermCommand->setEnabled(id == 0);
+    mXtermCommand->setText(id == 0 ? xtermCmd.second : QString());
     if (mUseAkonadi)
         mUseAkonadi->setChecked(Preferences::useAkonadi());
 }
 
-void MiscPrefTab::apply(bool syncToDisc)
+bool MiscPrefTab::apply(bool syncToDisc)
 {
     // First validate anything entered in Other X-terminal command
     int xtermID = mXtermType->selectedId();
-    if (xtermID >= mXtermCount)
+    if (!xtermID)
     {
+        // 'Other' is selected.
         QString cmd = mXtermCommand->text();
         if (cmd.isEmpty())
-            xtermID = -1;       // 'Other' is only acceptable if it's non-blank
+        {
+            mXtermCommand->setFocus();
+            const QList<QAbstractButton*> buttons = mXtermType->buttons();
+            if (buttons.count() == 2)
+            {
+                // There are only radio buttons for one standard command, plus 'Other'
+                const int id = mXtermType->id(buttons[0]);
+                cmd = Preferences::cmdXTermStandardCommand(id);
+                const QStringList args = KShell::splitArgs(cmd);
+                if (KAMessageBox::warningContinueCancel(topLayout()->parentWidget(), xi18nc("@info", "<para>No command is specified to invoke a terminal window.</para><para>The default command (<command>%1</command>) will be used.</para>", args[0]))
+                                != KMessageBox::Continue)
+                    return false;
+                xtermID = id;
+                mXtermType->setButton(id);
+            }
+            else
+            {
+                if (KAMessageBox::warningContinueCancel(topLayout()->parentWidget(), xi18nc("@info", "No command is specified to invoke a terminal window"))
+                                != KMessageBox::Continue)
+                    return false;
+                xtermID = -1;
+            }
+        }
         else
         {
             const QStringList args = KShell::splitArgs(cmd);
@@ -611,15 +613,10 @@ void MiscPrefTab::apply(bool syncToDisc)
                 mXtermCommand->setFocus();
                 if (KAMessageBox::warningContinueCancel(topLayout()->parentWidget(), xi18nc("@info", "Command to invoke terminal window not found: <command>%1</command>", cmd))
                                 != KMessageBox::Continue)
-                    return;
+                    return false;
             }
         }
     }
-    if (xtermID < 0)
-    {
-        xtermID = mXtermFirst;
-        mXtermType->setButton(mXtermFirst);
-    }
 
     if (mQuitWarn->isEnabled())
     {
@@ -645,16 +642,21 @@ void MiscPrefTab::apply(bool syncToDisc)
     int i = mDefaultDeferTime->value();
     if (i != Preferences::defaultDeferTime())
         Preferences::setDefaultDeferTime(i);
-    QString text = (xtermID < mXtermCount) ? xtermCommands[xtermID] : mXtermCommand->text();
-    if (text != Preferences::cmdXTermCommand())
-        Preferences::setCmdXTermCommand(text);
+    const auto xtermCmd = Preferences::cmdXTermCommandIndex();
+    if (xtermID != xtermCmd.first  ||  (!xtermID && mXtermCommand->text() != xtermCmd.second))
+    {
+        if (xtermID > 0)
+            Preferences::setCmdXTermCommand(xtermID);
+        else
+            Preferences::setCmdXTermSpecialCommand(mXtermCommand->text());
+    }
     if (mUseAkonadi)
     {
         b = mUseAkonadi->isChecked();
         if (b != Preferences::useAkonadi())
             Preferences::setUseAkonadi(b);
     }
-    PrefsTabBase::apply(syncToDisc);
+    return PrefsTabBase::apply(syncToDisc);
 }
 
 void MiscPrefTab::slotAutostartClicked()
@@ -663,11 +665,7 @@ void MiscPrefTab::slotAutostartClicked()
     &&  KAMessageBox::warningYesNo(topLayout()->parentWidget(),
                                    xi18nc("@info", "You should not uncheck this option unless you intend to discontinue use of <application>KAlarm</application>"),
                                    QString(), KStandardGuiItem::cont(), KStandardGuiItem::cancel()
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
                                   ) != KMessageBox::ButtonCode::PrimaryAction)
-#else
-                                  ) != KMessageBox::Yes)
-#endif
         mAutoStart->setChecked(true);
 }
 
@@ -859,7 +857,7 @@ void TimePrefTab::restore(bool, bool)
     mKOrgEventDuration->setValue(Preferences::kOrgEventDuration());
 }
 
-void TimePrefTab::apply(bool syncToDisc)
+bool TimePrefTab::apply(bool syncToDisc)
 {
     Preferences::setTimeSpec(mTimeZone->timeZone());
     const QString hol = mHolidays->itemData(mHolidays->currentIndex()).toString();
@@ -882,7 +880,7 @@ void TimePrefTab::apply(bool syncToDisc)
     t = mKOrgEventDuration->value();
     if (t != Preferences::kOrgEventDuration())
         Preferences::setKOrgEventDuration(t);
-    PrefsTabBase::apply(syncToDisc);
+    return PrefsTabBase::apply(syncToDisc);
 }
 
 
@@ -965,7 +963,7 @@ void StorePrefTab::restore(bool defaults, bool)
     mCheckKeepChanges = true;
 }
 
-void StorePrefTab::apply(bool syncToDisc)
+bool StorePrefTab::apply(bool syncToDisc)
 {
     const bool b = mAskResource->isChecked();
     if (b != Preferences::askResource())
@@ -973,7 +971,7 @@ void StorePrefTab::apply(bool syncToDisc)
     const int days = !mKeepArchived->isChecked() ? 0 : mPurgeArchived->isChecked() ? mPurgeAfter->value() : -1;
     if (days != Preferences::archivedKeepDays())
         Preferences::setArchivedKeepDays(days);
-    PrefsTabBase::apply(syncToDisc);
+    return PrefsTabBase::apply(syncToDisc);
 }
 
 void StorePrefTab::setArchivedControls(int purgeDays)
@@ -1164,7 +1162,7 @@ void EmailPrefTab::restore(bool defaults, bool)
         mAddressChanged = mBccAddressChanged = false;
 }
 
-void EmailPrefTab::apply(bool syncToDisc)
+bool EmailPrefTab::apply(bool syncToDisc)
 {
     const int client = mEmailClient->selectedId();
     if (client >= 0  &&  static_cast<Preferences::MailClient>(client) != Preferences::emailClient())
@@ -1188,7 +1186,7 @@ void EmailPrefTab::apply(bool syncToDisc)
     b = mEmailQueuedNotify->isChecked();
     if (b != Preferences::emailQueuedNotify())
         Preferences::setEmailQueuedNotify(mEmailQueuedNotify->isChecked());
-    PrefsTabBase::apply(syncToDisc);
+    return PrefsTabBase::apply(syncToDisc);
 }
 
 void EmailPrefTab::showEvent(QShowEvent* e)
@@ -1434,8 +1432,8 @@ EditPrefTab::EditPrefTab(StackedScrollGroup* scrollGroup)
     mSound->addItem(SoundPicker::i18n_combo_None());         // index 0
     mSound->addItem(SoundPicker::i18n_combo_Beep());         // index 1
     mSound->addItem(SoundPicker::i18n_combo_File());         // index 2
-#if KPIMTEXTEDIT_TEXT_TO_SPEECH
-    if (KPIMTextEdit::TextToSpeech::self()->isReady())
+#ifdef HAVE_TEXT_TO_SPEECH_SUPPORT
+    if (TextEditTextToSpeech::TextToSpeech::self()->isReady())
         mSound->addItem(SoundPicker::i18n_combo_Speak());    // index 3
 #endif
     mSound->setMinimumSize(mSound->sizeHint());
@@ -1582,7 +1580,7 @@ void EditPrefTab::restore(bool, bool allTabs)
     }
 }
 
-void EditPrefTab::apply(bool syncToDisc)
+bool EditPrefTab::apply(bool syncToDisc)
 {
     bool b = mAutoClose->isChecked();
     if (b != Preferences::defaultAutoClose())
@@ -1620,9 +1618,7 @@ void EditPrefTab::apply(bool syncToDisc)
     Preferences::SoundType snd;
     switch (mSound->currentIndex())
     {
-#if KPIMTEXTEDIT_TEXT_TO_SPEECH
         case 3:  snd = Preferences::Sound_Speak; break;
-#endif
         case 2:  snd = Preferences::Sound_File;  break;
         case 1:  snd = Preferences::Sound_Beep;  break;
         case 0:
@@ -1689,7 +1685,7 @@ void EditPrefTab::apply(bool syncToDisc)
     const QFont font = mFontChooser->font();
     if (font != Preferences::messageFont())
         Preferences::setMessageFont(font);
-    PrefsTabBase::apply(syncToDisc);
+    return PrefsTabBase::apply(syncToDisc);
 }
 
 void EditPrefTab::slotBrowseSoundFile()
@@ -1707,9 +1703,7 @@ int EditPrefTab::soundIndex(Preferences::SoundType type)
 {
     switch (type)
     {
-#if KPIMTEXTEDIT_TEXT_TO_SPEECH
         case Preferences::Sound_Speak: return 3;
-#endif
         case Preferences::Sound_File:  return 2;
         case Preferences::Sound_Beep:  return 1;
         case Preferences::Sound_None:
@@ -2004,7 +1998,7 @@ void ViewPrefTab::restore(bool, bool allTabs)
     }
 }
 
-void ViewPrefTab::apply(bool syncToDisc)
+bool ViewPrefTab::apply(bool syncToDisc)
 {
     QColor colour = mDisabledColour->color();
     if (colour != Preferences::disabledColour())
@@ -2053,7 +2047,7 @@ void ViewPrefTab::apply(bool syncToDisc)
         if (b != Preferences::modalMessages())
             Preferences::setModalMessages(b);
     }
-    PrefsTabBase::apply(syncToDisc);
+    return PrefsTabBase::apply(syncToDisc);
 }
 
 void ViewPrefTab::setTooltip(int maxAlarms, bool time, bool timeTo, const QString& prefix)
