@@ -13,18 +13,19 @@
 #include "pluginmanager.h"
 #include "lib/desktop.h"
 #include "lib/messagebox.h"
+#include "lib/shellprocess.h"
 #include "kalarmcalendar/identities.h"
 #include "kalarm_debug.h"
 
 #include <KIdentityManagement/Identity>
 #include <KIdentityManagement/IdentityManager>
 #include <KHolidays/HolidayRegion>
-#include <kpimtextedit/kpimtextedit-texttospeech.h>
 
 #include <KSharedConfig>
 #include <KConfigGroup>
 #include <KMessageBox>
 #include <KWindowSystem>
+#include <KShell>
 
 #include <QFile>
 #include <QSaveFile>
@@ -38,17 +39,42 @@ using namespace KAlarmCal;
 
 //clazy:excludeall=non-pod-global-static
 
+namespace
+{
+
 // Config file entry names
-static const char* GENERAL_SECTION  = "General";
+const char* GENERAL_SECTION  = "General";
 
 // Config file entry name for temporary use
-static const char* TEMP = "Temp";
+const char* TEMP = "Temp";
 
-static const QString AUTOSTART_FILE(QStringLiteral("kalarm.autostart.desktop"));
+const QString AUTOSTART_FILE(QStringLiteral("kalarm.autostart.desktop"));
 
 // Values for EmailFrom entry
-static const QString FROM_SYS_SETTINGS(QStringLiteral("@SystemSettings"));
-static const QString FROM_KMAIL(QStringLiteral("@KMail"));
+const QString FROM_SYS_SETTINGS(QStringLiteral("@SystemSettings"));
+const QString FROM_KMAIL(QStringLiteral("@KMail"));
+
+// Command strings for executing commands in different types of terminal windows.
+// %t = window title parameter
+// %c = command to execute in terminal
+// %w = command to execute in terminal, with 'sleep 86400' appended
+// %C = temporary command file to execute in terminal
+// %W = temporary command file to execute in terminal, with 'sleep 86400' appended
+const QVector<QString> xtermCommands {
+    QStringLiteral("xterm -sb -hold -title %t -e %c"),
+    QStringLiteral("konsole --noclose -p tabtitle=%t -e ${SHELL:-sh} -c %c"),
+    QStringLiteral("gnome-terminal -t %t -e %W"),
+    QStringLiteral("eterm --pause -T %t -e %C"),    // some systems use eterm...
+    QStringLiteral("Eterm --pause -T %t -e %C"),    // while some use Eterm
+    QStringLiteral("rxvt -title %t -e ${SHELL:-sh} -c %w"),
+    QStringLiteral("xfce4-terminal -T %t -H -e %c")
+};
+
+QVector<QString> xtermCommandExes;   // initialised to hold executables in xtermCommands
+
+void splitXTermCommands();
+
+} // namespace
 
 // Config file entry names for notification messages
 const QLatin1String Preferences::QUIT_WARN("QuitWarn");
@@ -499,17 +525,93 @@ void Preferences::setEmailBccAddress(bool useSystemSettings, const QString& addr
 
 QString Preferences::cmdXTermCommand()
 {
-    return translateXTermPath(self()->mBase_CmdXTermCommand, false);
+    QString cmd = translateXTermPath(self()->mBase_CmdXTermCommand, false);
+    if (cmd.isEmpty())
+    {
+        // No terminal command is specified. If there is only one standard
+        // terminal executable installed, set it as the default.
+        const auto cmds = cmdXTermStandardCommands();
+        if (cmds.size() == 1)
+        {
+            cmd = cmds.constBegin().value();
+            self()->setBase_CmdXTermCommand(translateXTermPath(cmd, true));
+            self()->save();
+        }
+    }
+    return cmd;
 }
 
-void Preferences::setCmdXTermCommand(const QString& cmd)
+std::pair<int, QString> Preferences::cmdXTermCommandIndex()
+{
+    const QString xtermCmd = cmdXTermCommand();
+    if (xtermCmd.isEmpty())
+        return std::make_pair(-1, QString());
+    int id = 0;
+    for (const QString& cmd : xtermCommands)
+    {
+        ++id;
+        if (xtermCmd == cmd)
+            return std::make_pair(id, xtermCmd);   // index is 1 greater than the index into xtermCommands
+    }
+    return std::make_pair(0, xtermCmd);
+}
+
+void Preferences::setCmdXTermSpecialCommand(const QString& cmd)
 {
     self()->setBase_CmdXTermCommand(translateXTermPath(cmd, true));
 }
 
+void Preferences::setCmdXTermCommand(int index)
+{
+    --index;   // convert to an index into xtermCommands
+    if (index >= 0  &&  index < xtermCommands.size())
+        setCmdXTermSpecialCommand(xtermCommands[index]);
+}
+
+QString Preferences::cmdXTermStandardCommand(int index)
+{
+    --index;   // convert to an index into xtermCommands
+    if (index >= 0  &&  index < xtermCommands.size())
+    {
+        splitXTermCommands();
+        const QString exe = xtermCommandExes.at(index);
+        if (!exe.isEmpty()  &&  !QStandardPaths::findExecutable(exe).isEmpty())
+            return xtermCommands[index];
+    }
+    return {};
+}
+
+QHash<int, QString> Preferences::cmdXTermStandardCommands()
+{
+    QHash<int, QString> result;
+    splitXTermCommands();
+    for (int i = 0, count = xtermCommands.count();  i < count;  ++i)
+    {
+        const QString exe = xtermCommandExes.at(i);
+        if (!exe.isEmpty()  &&  !QStandardPaths::findExecutable(exe).isEmpty())
+            result[i + 1] = xtermCommands[i];   // index is 1 greater than the index into xtermCommands
+    }
+    return result;
+}
+
+namespace
+{
+void splitXTermCommands()
+{
+    if (xtermCommandExes.isEmpty())
+    {
+        for (const QString& cmd : xtermCommands)
+        {
+            const QStringList args = KShell::splitArgs(cmd);
+            xtermCommandExes.append(args.isEmpty() ? QString() : args[0]);
+        }
+    }
+}
+}
+
 Preferences::SoundType Preferences::defaultSoundType()
 {
-#if KPIMTEXTEDIT_TEXT_TO_SPEECH
+#ifdef HAVE_TEXT_TO_SPEECH_SUPPORT
     return self()->base_DefaultSoundType();
 #else
     SoundType type = self()->base_DefaultSoundType();
@@ -519,7 +621,7 @@ Preferences::SoundType Preferences::defaultSoundType()
 
 void Preferences::setDefaultSoundType(SoundType type)
 {
-#if !KPIMTEXTEDIT_TEXT_TO_SPEECH
+#ifndef HAVE_TEXT_TO_SPEECH_SUPPORT
     if (type == Sound_Speak)
         return;
 #endif
@@ -563,40 +665,9 @@ bool Preferences::notifying(const QString& messageID)
 */
 QString translateXTermPath(const QString& cmdline, bool write)
 {
-    QString params;
-    QString cmd = cmdline;
-    if (cmdline.isEmpty())
-        return cmdline;
-    // Strip any leading quote
-    const QChar quote = cmdline[0];
-    const char q = quote.toLatin1();
-    bool quoted = (q == '"' || q == '\'');
-    // Split the command at the first non-escaped space
-    for (int i = quoted ? 1 : 0, count = cmd.length();  i < count;  ++i)
-    {
-        switch (cmd.at(i).toLatin1())
-        {
-            case '\\':
-                ++i;
-                continue;
-            case '"':
-            case '\'':
-                if (quoted  &&  cmd.at(i) == quote)
-                    quoted = false;
-                continue;
-            case ' ':
-                if (!quoted)
-                {
-                    params = cmd.mid(i);
-                    cmd.truncate(i);
-                    break;
-                }
-                continue;
-            default:
-                continue;
-        }
-        break;
-    }
+    QString params = cmdline;
+    QString cmd = ShellProcess::splitCommandLine(params);
+
     // Translate any home directory specification at the start of the
     // executable's path.
     KConfigGroup group(KSharedConfig::openConfig(), GENERAL_SECTION);
@@ -611,10 +682,7 @@ QString translateXTermPath(const QString& cmdline, bool write)
         cmd = group.readPathEntry(TEMP, QString());
     }
     group.deleteEntry(TEMP);
-    if (quoted)
-        return quote + cmd + params;
-    else
-        return cmd + params;
+    return cmd + params;
 }
 
 // vim: et sw=4:
