@@ -11,8 +11,6 @@
 #include "resources.h"
 #include "singlefileresource.h"
 #include "fileresourcecalendarupdater.h"
-#include "preferences.h"
-#include "lib/messagebox.h"
 #include "kalarm_debug.h"
 
 #include <KConfig>
@@ -57,17 +55,14 @@ void FileResourceConfigManager::createResources(QObject* parent)
         return;
     manager->mCreated = 1;
 
-    QStringList resourceGroups = manager->mConfig->groupList().filter(QRegularExpression(QStringLiteral("^Resource_\\d+$")));
+    static const QRegularExpression re(QStringLiteral("^Resource_\\d+$"));
+    QStringList resourceGroups = manager->mConfig->groupList().filter(re);
     if (!resourceGroups.isEmpty())
     {
         std::sort(resourceGroups.begin(), resourceGroups.end(),
                   [](const QString& g1, const QString& g2)
                   {
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-                      return g1.midRef(9).toInt() < g2.midRef(9).toInt();
-#else
                       return QStringView(g1).mid(9).toInt() < QStringView(g2).mid(9).toInt();
-#endif
                   });
 
         KConfigGroup general(manager->mConfig, GROUP_GENERAL);
@@ -75,11 +70,7 @@ void FileResourceConfigManager::createResources(QObject* parent)
 
         for (const QString& resourceGroup : std::as_const(resourceGroups))
         {
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-            const int groupIndex = resourceGroup.midRef(9).toInt();
-#else
             const int groupIndex = QStringView(resourceGroup).mid(9).toInt();
-#endif
             FileResourceSettings::Ptr settings(new FileResourceSettings(manager->mConfig, resourceGroup));
             if (!settings->isValid())
             {
@@ -252,7 +243,7 @@ Resource FileResourceConfigManager::createResource(FileResourceSettings::Ptr& se
     switch (settings->storageType())
     {
         case FileResourceSettings::File:
-            return SingleFileResource::create(settings.data());
+            return SingleFileResource::create(settings);
         case FileResourceSettings::Directory:   // not currently intended to be implemented
         default:
             return Resource::null();
