@@ -14,11 +14,9 @@
 #include "lib/file.h"
 #include "lib/pushbutton.h"
 
-#include <kpimtextedit/kpimtextedit-texttospeech.h>
-#if KPIMTEXTEDIT_TEXT_TO_SPEECH
-#include <KPIMTextEdit/TextToSpeech>
+#ifdef HAVE_TEXT_TO_SPEECH_SUPPORT
+#include <TextEditTextToSpeech/TextToSpeech>
 #endif
-
 #include <KLocalizedString>
 #include <phonon/backendcapabilities.h>
 
@@ -131,9 +129,8 @@ void SoundPicker::showFile(bool show)
 */
 void SoundPicker::showSpeak(bool show)
 {
-#if KPIMTEXTEDIT_TEXT_TO_SPEECH
-    if (!KPIMTextEdit::TextToSpeech::self()->isReady())
-#endif
+#ifdef HAVE_TEXT_TO_SPEECH_SUPPORT
+    if (!TextEditTextToSpeech::TextToSpeech::self()->isReady())
         show = false;    // speech capability is not installed or configured
     if (show != mSpeakShowing)
     {
@@ -148,6 +145,7 @@ void SoundPicker::showSpeak(bool show)
         }
         mSpeakShowing = show;
     }
+#endif
 }
 
 /******************************************************************************
@@ -157,10 +155,6 @@ Preferences::SoundType SoundPicker::sound() const
 {
     if (mTypeCombo->currentIndex() < 0)
         return Preferences::Sound_None;
-#if !KPIMTEXTEDIT_TEXT_TO_SPEECH
-    if (mTypeCombo->currentData().toInt() == Preferences::Sound_Speak)
-        return Preferences::Sound_None;
-#endif
     return static_cast<Preferences::SoundType>(mTypeCombo->currentData().toInt());
 }
 
